@@ -1,3 +1,23 @@
+"""
+eels_tools
+Model based quantification of electron energy-loss data
+Copyright by Gerd Duscher
+
+The University of Tennessee, Knoxville
+Department of Materials Science & Engineering
+
+Sources:
+   M. Tian et al.
+
+Units:
+    everything is in SI units, except length is given in nm and angles in mrad.
+
+Usage:
+    See the notebooks for examples of these routines
+
+All the input and output is done through a dictionary which is to be found in the meta_data
+attribute of the sidpy.Dataset
+"""
 import numpy as np
 
 import scipy
@@ -54,6 +74,8 @@ elements = [' ', 'H', 'He', 'Li', 'Be', 'B', 'C', 'N', 'O', 'F', 'Ne', 'Na',
 
 
 def read_dm3_eels_info(original_metadata):
+    """Reed dm3 file from a nested dictionary like original_metadata of sidpy.Dataset"""
+
     if 'DM' not in original_metadata:
         return {}
     main_image = original_metadata['DM']['chosen_image']
@@ -90,6 +112,8 @@ def read_dm3_eels_info(original_metadata):
 
 
 def set_previous_quantification(current_dataset):
+    """Set previous qunatifications from a sidpy.Dataset"""
+
     current_channel = current_dataset.h5_dataset.parent
     found_metadata = False
     for key in current_channel:
@@ -119,11 +143,15 @@ def set_previous_quantification(current_dataset):
 
 
 def residuals_smooth(p, x, y, only_positive_intensity):
+    """part of fit"""
+
     err = (y - model_smooth(x, p, only_positive_intensity))
     return err
 
 
 def model_smooth(x, p, only_positive_intensity=False):
+    """part of fit"""
+
     y = np.zeros(len(x))
 
     number_of_peaks = int(len(p) / 3)
@@ -140,16 +168,22 @@ def model_smooth(x, p, only_positive_intensity=False):
 
 
 def residuals_ll(p, x, y, only_positive_intensity):
+    """part of fit"""
+
     err = (y - model_ll(x, p, only_positive_intensity)) / np.sqrt(np.abs(y))
     return err
 
 
 def residuals_ll2(p, x, y, only_positive_intensity):
+    """part of fit"""
+
     err = (y - model_ll(x, p, only_positive_intensity))
     return err
 
 
 def model_ll(x, p, only_positive_intensity):
+    """part of fit"""
+
     y = np.zeros(len(x))
 
     number_of_peaks = int(len(p) / 3)
@@ -166,6 +200,8 @@ def model_ll(x, p, only_positive_intensity):
 
 
 def fit_peaks(spectrum, energy_scale, pin, start_fit, end_fit, only_positive_intensity=False):
+    """fit peaks to spectrum"""
+
     # TODO: remove zero_loss_fit_width add absolute
 
     fit_energy = energy_scale[start_fit:end_fit]
@@ -189,13 +225,17 @@ def fit_peaks(spectrum, energy_scale, pin, start_fit, end_fit, only_positive_int
 
 
 def get_x_sections(z=0):
-    """
-    ####
-    # reads X-ray fluorescent cross sections from a pickle file.
-    ####
-    Input: nothing or atomic number
-    Output: dictionary
-            of a element or of all elements if z = 0
+    """Reads X-ray fluorescent cross sections from a pickle file.
+
+    Parameters
+    ----------
+    z: int
+        atomic number if zero all cross sections will be returned
+
+    Returns
+    -------
+    dictionary
+        cross section of a element or of all elements if z = 0
 
     """
     pkl_file = open(data_path + '/edges_db.pkl', 'rb')
@@ -215,11 +255,12 @@ def get_x_sections(z=0):
 
 
 def get_z(z):
-    """
-    returns the atomic number independent of input as a string or number
+    """Returns the atomic number independent of input as a string or number
 
-    input:
-    z: atomic number of chemical symbol (0 if not valid)
+    Parameter
+    ---------
+    z: int, str
+        atomic number of chemical symbol (0 if not valid)
     """
     x_sections = get_x_sections()
 
@@ -234,6 +275,8 @@ def get_z(z):
 
 
 def list_all_edges(z):
+    """List all ionization edges of an element with atomic number z"""
+
     element = str(z)
     x_sections = get_x_sections()
     print('Major edges')
@@ -243,7 +286,22 @@ def list_all_edges(z):
                 print(f" {x_sections[element]['name']}-{key}: {x_sections[element][key]['onset']:8.1f} eV ")
 
 
-def find_major_edges(edge_onset, maximal_chemical_shift=5):
+def find_major_edges(edge_onset, maximal_chemical_shift=5.):
+    """Find all major edges within an energy range
+
+    Parameters
+    ----------
+    edge_onset: float
+        approximate energy of ionization edge
+    maximal_chemical_shift: float
+        optional, range of energy window around edge_onset to look for major edges
+
+    Returns
+    -------
+    text: str
+        string with all major edges in energy range
+
+    """
     text = ''
     x_sections = get_x_sections()
     for element in x_sections:
@@ -261,6 +319,22 @@ def find_major_edges(edge_onset, maximal_chemical_shift=5):
 
 
 def find_all_edges(edge_onset, maximal_chemical_shift=5):
+    """Find all (major and minor) edges within an energy range
+
+    Parameters
+    ----------
+    edge_onset: float
+        approximate energy of ionization edge
+    maximal_chemical_shift: float
+        optional, range of energy window around edge_onset to look for major edges
+
+    Returns
+    -------
+    text: str
+        string with all edges in energy range
+
+    """
+
     text = ''
     x_sections = get_x_sections()
     for element in x_sections:
@@ -276,6 +350,8 @@ def find_all_edges(edge_onset, maximal_chemical_shift=5):
 
 
 def second_derivative(dataset, sensitivity):
+    """Calculates second derivative of a sidpy.dataset"""
+
     dim = ft.get_dimensions_by_type('spectral', dataset)
     energy_scale = np.array(dim[0][1])
     if dataset.data_type.name == 'SPECTRAL_IMAGE':
@@ -309,6 +385,8 @@ def second_derivative(dataset, sensitivity):
 
 
 def find_edges(dataset, sensitivity=2.5):
+    """find edges within a sidpy.Dataset"""
+
     dim = ft.get_dimensions_by_type('spectral', dataset)
     energy_scale = np.array(dim[0][1])
 
@@ -360,8 +438,23 @@ def find_edges(dataset, sensitivity=2.5):
 
 
 def make_edges(edges_present, energy_scale, e_0, coll_angle):
-    """
-    Makes the edges dictionary
+    """Makes the edges dictionary for quantification
+
+    Parameters
+    ----------
+    edges_present: list
+        list of edges
+    energy_scale: numpy array
+        energy scale on which to make cross section
+    e_0: float
+        acceleration voltage (in V)
+    coll_angle: float
+        collection angle in mrad
+
+    Returns
+    -------
+    edges: dict
+        dictionary with all information on cross section
     """
     x_sections = get_x_sections()
     edges = {}
@@ -402,8 +495,7 @@ def make_edges(edges_present, energy_scale, e_0, coll_angle):
 
 
 def make_cross_sections(edges, energy_scale, e_0, coll_angle):
-    """
-    Updates the edges dictionary with collection angle-integrated X-ray photoabsorption cross-sections
+    """Updates the edges dictionary with collection angle-integrated X-ray photoabsorption cross-sections
     """
     for key in edges:
         if key.isdigit():
@@ -417,10 +509,12 @@ def make_cross_sections(edges, energy_scale, e_0, coll_angle):
 
 
 def power_law(energy, a, r):
+    """power law for power_law_background"""
     return a * np.power(energy, -r)
 
 
 def power_law_background(spectrum, energy_scale, fit_area, verbose=False):
+    """fit of power law to spectrum """
     # Determine energy window  for background fit in pixels
 
     startx = np.searchsorted(energy_scale, fit_area[0])
@@ -454,6 +548,7 @@ def power_law_background(spectrum, energy_scale, fit_area, verbose=False):
 
 
 def cl_model(x, p, number_of_edges, xsec):
+    """ core loss model for fitting"""
     y = (p[9] * np.power(x, (-p[10]))) + p[7] * x + p[8] * x * x
     for i in range(number_of_edges):
         y = y + p[i] * xsec[i, :]
@@ -461,6 +556,8 @@ def cl_model(x, p, number_of_edges, xsec):
 
 
 def fit_edges2(spectrum, energy_scale, edges):
+    """fit edges for quantification"""
+
     dispersion = energy_scale[1] - energy_scale[0]
     # Determine fitting ranges and masks to exclude ranges
     mask = np.ones(len(spectrum))
@@ -547,6 +644,8 @@ def fit_edges2(spectrum, energy_scale, edges):
 
 
 def fit_edges(spectrum, energy_scale, region_tags, edges):
+    """fit edges for quantification"""
+
     # Determine fitting ranges and masks to exclude ranges
     mask = np.ones(len(spectrum))
 
@@ -625,6 +724,8 @@ def fit_edges(spectrum, energy_scale, region_tags, edges):
 
 
 def find_peaks(dataset, fit_start, fit_end, sensitivity=2):
+    """find peaks in spectrum"""
+
     if dataset.data_type.name == 'SPECTRAL_IMAGE':
         spectrum = dataset.view.get_spectrum()
     else:
@@ -671,15 +772,20 @@ def find_peaks(dataset, fit_start, fit_end, sensitivity=2):
 
 
 def find_maxima(y, number_of_peaks):
-    """
-    find the first most prominent peaks
+    """ find the first most prominent peaks
+
     peaks are then sorted by energy
 
-    input:
-        y: array of (part) of spectrum
-        number_of_peaks: int
-    output:
-        array of indices of peaks
+    Parameters
+    ----------
+    y: numpy array
+        (part) of spectrum
+    number_of_peaks: int
+
+    Returns
+    -------
+    numpy array
+        indices of peaks
     """
     blurred2 = gaussian_filter(y, sigma=2)
     peaks, _ = scipy.signal.find_peaks(blurred2)
@@ -692,7 +798,9 @@ def find_maxima(y, number_of_peaks):
 
 
 def gauss(x, p):  # p[0]==mean, p[1]= amplitude p[2]==fwhm,
-    """
+    """Gaussian Function
+
+        p[0]==mean, p[1]= amplitude p[2]==fwhm
         area = np.sqrt(2* np.pi)* p[1] * np.abs(p[2] / 2.3548)
         FWHM = 2 * np.sqrt(2 np.log(2)) * sigma = 2.3548 * sigma
         sigma = FWHM/3548
@@ -704,11 +812,13 @@ def gauss(x, p):  # p[0]==mean, p[1]= amplitude p[2]==fwhm,
 
 
 def lorentz(x, p):
+    """lorentzian function"""
     lorentz_peak = 0.5 * p[2] / np.pi / ((x - p[0]) ** 2 + (p[2] / 2) ** 2)
     return p[1] * lorentz_peak / lorentz_peak.max()
 
 
 def zl(x, p, p_zl):
+    """zero-loss function"""
     p_zl_local = p_zl.copy()
     p_zl_local[2] += p[0]
     p_zl_local[5] += p[0]
@@ -717,6 +827,7 @@ def zl(x, p, p_zl):
 
 
 def model3(x, p, number_of_peaks, peak_shape, p_zl, pin=None, restrict_pos=0, restrict_width=0):
+    """ model for fitting low-loss spectrum"""
     if pin is None:
         pin = p
 
@@ -752,6 +863,7 @@ def model3(x, p, number_of_peaks, peak_shape, p_zl, pin=None, restrict_pos=0, re
 
 
 def sort_peaks(p, peak_shape):
+    """sort fitting parameters by peak position"""
     number_of_peaks = int(len(p) / 3)
     p3 = np.reshape(p, (number_of_peaks, 3))
     sort_pin = np.argsort(p3[:, 0])
@@ -763,6 +875,7 @@ def sort_peaks(p, peak_shape):
 
 
 def add_peaks(x, y, peaks, pin_in=None, peak_shape_in=None, shape='Gaussian'):
+    """ add peaks to fitting parameters"""
     if pin_in is None:
         return
     if peak_shape_in is None:
@@ -786,6 +899,8 @@ def add_peaks(x, y, peaks, pin_in=None, peak_shape_in=None, shape='Gaussian'):
 
 
 def fit_model(x, y, pin, number_of_peaks, peak_shape, p_zl, restrict_pos=0, restrict_width=0):
+    """model for fitting low-loss spectrum"""
+
     pin_original = pin.copy()
 
     def residuals3(pp, xx, yy):
@@ -805,6 +920,7 @@ def fit_model(x, y, pin, number_of_peaks, peak_shape, p_zl, restrict_pos=0, rest
 
 
 def fix_energy_scale(spec, energy):
+    """Shift energy scale according to zero-loss peak position"""
     # determine start and end fitting region in pixels
     start = np.searchsorted(energy, -10)
     end = np.searchsorted(energy, 10)
@@ -838,6 +954,8 @@ def fix_energy_scale(spec, energy):
 
 
 def resolution_function(energy_scale, spectrum, width, verbose=False):
+    """get resolution function (zero-loss peak shape) from low-loss spectrum"""
+
     guess = [0.2, 1000, 0.02, 0.2, 1000, 0.2]
     p0 = np.array(guess)
 
@@ -898,6 +1016,7 @@ def resolution_function(energy_scale, spectrum, width, verbose=False):
 
 
 def get_energy_shifts(spectrum_image, energy_scale, zero_loss_fit_width):
+    """get shift of spectrum form zero-loss peak position"""
     shifts = np.zeros(spectrum_image.shape[0:2])
     for x in range(spectrum_image.shape[0]):
         for y in range(spectrum_image.shape[1]):
@@ -910,6 +1029,8 @@ def get_energy_shifts(spectrum_image, energy_scale, zero_loss_fit_width):
 
 
 def shift_on_same_scale(spectrum_image, shift, energy_scale, master_energy_scale):
+    """shift spectrum in energy"""
+
     new_si = np.zeros(spectrum_image.shape)
     for x in range(spectrum_image.shape[0]):
         for y in range(spectrum_image.shape[1]):
@@ -919,16 +1040,22 @@ def shift_on_same_scale(spectrum_image, shift, energy_scale, master_energy_scale
 
 
 def get_wave_length(e0):
+    """get deBroglie wavelength of electron accelerated by energy (in eV) e0"""
+
     ev = constants.e * e0
     return constants.h / np.sqrt(2 * constants.m_e * ev * (1 + ev / (2 * constants.m_e * constants.c ** 2)))
 
 
 def drude(ep, eb, gamma, e):
+    """dielectric function according to Drude theory"""
+
     eps = 1 - (ep ** 2 - eb * e * 1j) / (e ** 2 + 2 * e * gamma * 1j)  # Mod drude term
     return eps
 
 
 def drude_lorentz(eps_inf, leng, ep, eb, gamma, e, amplitude):
+    """dielectric function according to Drude-Lorentz theory"""
+
     eps = eps_inf
     for i in range(leng):
         eps = eps + amplitude[i] * (1 / (e + ep[i] + gamma[i] * 1j) - 1 / (e - ep[i] + gamma[i] * 1j))
@@ -936,6 +1063,8 @@ def drude_lorentz(eps_inf, leng, ep, eb, gamma, e, amplitude):
 
 
 def plot_dispersion(plotdata, units, a_data, e_data, title, max_p, ee, ef=4., ep=16.8, es=0, ibt=[]):
+    """Plot loss function """
+
     [x, y] = np.meshgrid(e_data + 1e-12, a_data[1024:2048] * 1000)
 
     z = plotdata
@@ -994,6 +1123,8 @@ def plot_dispersion(plotdata, units, a_data, e_data, title, max_p, ee, ef=4., ep
 
 
 def zl_func(p, x):
+    """zero-loss peak function"""
+
     p[0] = abs(p[0])
 
     gauss1 = np.zeros(len(x))
@@ -1011,20 +1142,29 @@ def zl_func(p, x):
 
 
 def drude2(tags, e, p):
+    """dielectric function according to Drude theory for fitting"""
+
     return drude(e, p[0], p[1], p[2], p[3])
 
 
 def xsec_xrpa(energy_scale, e0, z, beta, shift=0):
-    """
-    Calculate momentum-integrated cross-section for EELS from X-ray photoaborption  cross-sections.
+    """ Calculate momentum-integrated cross-section for EELS from X-ray photo-absorption cross-sections.
 
-    Input:
-    ------
-    energy_scale: energyscale of spectrum to be analyzed
-    e0: acceleration voltage in keV
-    z: atomic number of element
-    beta: effective collection angle in mrad
-    shift: chemical shift of edge in eV
+    X-ray photo-absorption cross-sections from NIST.
+    Momentum-integrated cross-section for EELS according to Egerton Ultramicroscopy 50 (1993) 13-28 equation (4)
+
+    Parameters
+    ----------
+    energy_scale: numpy array
+        energy scale of spectrum to be analyzed
+    e0: float
+        acceleration voltage in keV
+    z: int
+        atomic number of element
+    beta: float
+        effective collection angle in mrad
+    shift: float
+        chemical shift of edge in eV
     """
     beta = beta * 0.001  # collection half angle theta [rad]
     # thetamax = self.parent.spec[0].convAngle * 0.001  # collection half angle theta [rad]
@@ -1061,7 +1201,8 @@ def xsec_xrpa(energy_scale, e0, z, beta, shift=0):
 
 
 def drude_simulation(dset, e, ep, ew, tnm, eb):
-    """
+    """probabilities of dielectric function eps relative to zero-loss integral (i0 = 1)
+
     Gives probabilities of dielectric function eps relative to zero-loss integral (i0 = 1) per eV
     Details in R.F.Egerton: EELS in the Electron Microscope, 3rd edition, Springer 2011)
 
@@ -1176,7 +1317,8 @@ def drude_simulation(dset, e, ep, ew, tnm, eb):
 
 
 def effective_collection_angle(energy_scale, alpha, beta, beam_kv):
-    """
+    """Calculates the effective collection angle in mrad:
+
     Translate from original Fortran program
     Calculates the effective collection angle in mrad:
     Input:
@@ -1297,9 +1439,8 @@ def effective_collection_angle(energy_scale, alpha, beta, beam_kv):
 
 
 def kroeger_core(e_data, a_data, eps_data, ee, thick, relativistic=True):
-    """
+    """This function calculates the differential scattering probability
 
-    This function calculates the differential scattering probability
      .. math::
         \\frac{d^2P}{d \\Omega d_e}
     of the low-loss region for total loss and volume plasmon loss
@@ -1431,9 +1572,8 @@ def kroeger_core(e_data, a_data, eps_data, ee, thick, relativistic=True):
 
 
 def kroeger_core2(e_data, a_data, eps_data, acceleration_voltage_kev, thickness, relativistic=True):
-    """
+    """This function calculates the differential scattering probability
 
-    This function calculates the differential scattering probability
      .. math::
         \\frac{d^2P}{d \\Omega d_e}
     of the low-loss region for total loss and volume plasmon loss
@@ -1452,6 +1592,7 @@ def kroeger_core2(e_data, a_data, eps_data, acceleration_voltage_kev, thickness,
 
        return P, P*scale*1e2,p_vol*1e2, p_simple*1e2
     """
+
     # $d^2P/(dEd\Omega) = \frac{1}{\pi^2 a_0 m_0 v^2} \Im \left[ \frac{t\mu^2}{\varepsilon \phi^2 } \right]
     """
     # Internally everything is calculated in si units
@@ -1577,6 +1718,7 @@ def kroeger_core2(e_data, a_data, eps_data, acceleration_voltage_kev, thickness,
 
 
 def read_msa(msa_string):
+    """read msa formated file"""
     parameters = {}
     y = []
     x = []
