@@ -1,3 +1,5 @@
+"""file_tools: All tools to load and save data
+"""
 ##################################
 #
 # 2018 01 31 Included Nion Swift files to be opened
@@ -8,6 +10,22 @@
 import numpy as np
 import h5py
 import os
+import sys
+
+
+# =============================================
+#   Include sidpy and other pyTEMlib Libraries                                      #
+# =============================================
+from .config_dir import config_path
+
+# from .nsi_reader import NSIDReader
+from .dm3_reader import DM3Reader
+from .nion_reader import NionReader
+import pyNSID
+import sidpy
+
+
+from .sidpy_tools import *
 
 # Open/Save File dialog
 try:
@@ -16,25 +34,11 @@ try:
 except ImportError:
     QT_available = False
 
-# =============================================================
-#   Include sidpy and other pyTEMlib Libraries                                      #
-# =============================================================
-from .config_dir import config_path
-
-from .nsi_reader import NSIDReader
-from .dm3_reader import DM3Reader
-from .nion_reader import NionReader
-import pyNSID
-
-import ipywidgets as widgets
-from IPython.display import display
-from .sidpy_tools import *
+if "google.colab" in sys.modules:
+    QT_available = False
 
 if QT_available:
     from .qt_sidpy_tools import *
-# import sys
-# sys.path.insert(0, "../../sidpy/")
-import sidpy
 
 
 Dimension = sidpy.Dimension
@@ -43,30 +47,136 @@ nest_dict = sidpy.base.dict_utils.nest_dict
 get_slope = sidpy.base.num_utils.get_slope
 __version__ = '10.30.2020'
 
-# TODO: new sidpy-version, uncomment and delete function below.
-# flatten_dict = sidpy.dict_utils.flatten_dict
+flatten_dict = sidpy.dict_utils.flatten_dict
 
 
-def flatten_dict(d, parent_key='', sep='-'):
-    items = []
-    for k, v in d.items():
-        if sep in k:
-            k = k.replace(sep, '_')
-        new_key = parent_key + sep + k if parent_key else k
-        if isinstance(v, dict):
-            items.extend(flatten_dict(v, new_key, sep=sep).items())
-        elif isinstance(v, list):
-            for i in range(len(v)):
-                if isinstance(v[i], dict):
-                    for kk in v[i]:
-                        items.append(('dim-'+kk+'-'+str(i), v[i][kk]))
-                else:
-                    if type(v) != bytes:
-                        items.append((new_key, v))
+class FileWidget(object):
+    """Widget to select directories or widgets from a list
+
+    Works in google colab.
+    The widgets converts the name of the nion file to the one in Nion's swift software,
+    because it is otherwise incomprehensible
+
+    Attributes
+    ----------
+    dir_name: str
+        name of starting directory
+    extension: list of str
+        extensions of files to be listed  in widget
+
+    Methods
+    -------
+    get_directory
+    set_options
+    get_file_name
+
+    Example
+    -------
+    >>from google.colab import drive
+    >>drive.mount("/content/drive")
+    >>file_list = pyTEMlib.file_tools.FileWidget()
+    next code cell:
+    >>dataset = pyTEMlib.file_tools.open_file(file_list.file_name)
+
+    """
+
+    def __init__(self, dir_name=None, extension=['*']):
+        self.save_path = False
+        self.dir_dictionary = {}
+        self.dir_list = ['.', '..']
+        self.display_list = ['.', '..']
+
+        if dir_name is None:
+            dir_name = get_last_path()
+            self.save_path = True
+            self.get_directory(dir_name)
+        elif os.path.isdir(dir_name):
+            self.get_directory(dir_name)
+            self.dir_name = dir_name
         else:
-            if type(v) != bytes:
-                items.append((new_key, v))
-    return dict(items)
+            self.dir_name = '.'
+            self.get_directory(self.dir_name)
+
+        self.dir_list == ['.']
+        self.extensions = extension
+        self.file_name = ''
+
+        self.select_files = widgets.Select(
+            options=self.dir_list,
+            value=self.dir_list[0],
+            description='Select file:',
+            disabled=False,
+            rows=10,
+            layout=widgets.Layout(width='70%')
+        )
+        display(self.select_files)
+        self.set_options()
+        self.select_files.observe(self.get_file_name, names='value')
+
+    def get_directory(self, directory=None):
+        self.dir_name = directory
+        self.dir_dictionary = {}
+        self.dir_list = []
+        self.dir_list = ['.', '..'] + os.listdir(directory)
+
+    def set_options(self):
+        self.dir_name = os.path.abspath(os.path.join(self.dir_name, self.dir_list[self.select_files.index]))
+        dir_list = os.listdir(self.dir_name)
+
+        file_list = []
+        display_file_list = []
+        directory_list = []
+
+        for i in range(len(dir_list)):
+            name = dir_list[i]
+            full_name = os.path.join(self.dir_name, name)
+
+            if os.path.isfile(full_name):
+                size = os.path.getsize(full_name) * 2 ** -20
+                basename, extension = os.path.splitext(name)
+                if self.extensions[0] == 'hf5':
+                    if extension in ['.hf5']:
+                        file_list.append(dir_list[i])
+                        display_file_list.append(f" {name}  - {size:.1f} MB")
+                else:
+                    file_list.append(dir_list[i])
+                    if extension in ['.h5', '.ndata']:
+                        reader = NionReader(full_name)
+                        dataset_nion = reader.read()
+                        display_file_list.append(f" {dataset_nion.title}{extension}  - {size:.1f} MB")
+                    elif extension in ['.hf5']:
+                        display_file_list.append(f" {name}  -- {size:.1f} MB")
+                    else:
+                        display_file_list.append(f' {name}  - {size:.1f} MB')
+            else:
+                directory_list.append(name)
+
+        sort = np.argsort(directory_list)
+        self.dir_list = ['.', '..']
+        self.display_list = ['.', '..']
+        for j in sort:
+            self.display_list.append(f' * {directory_list[j]}')
+            self.dir_list.append(directory_list[j])
+
+        sort = np.argsort(display_file_list)
+
+        for i, j in enumerate(sort):
+            if '--' in dir_list[j]:
+                self.display_list.append(f' {i:3} {display_file_list[j]}')
+            else:
+                self.display_list.append(f' {i:3}   {display_file_list[j]}')
+            self.dir_list.append(file_list[j])
+
+        self.dir_label = os.path.split(self.dir_name)[-1] + ':'
+        self.select_files.options = self.display_list
+
+    def get_file_name(self, b):
+
+        if os.path.isdir(os.path.join(self.dir_name, self.dir_list[self.select_files.index])):
+            self.set_options()
+
+        elif os.path.isfile(os.path.join(self.dir_name, self.dir_list[self.select_files.index])):
+            self.file_name = os.path.join(self.dir_name, self.dir_list[self.select_files.index])
 
 
 ####
@@ -75,12 +185,7 @@ def flatten_dict(d, parent_key='', sep='-'):
 
 
 def get_qt_app():
-    """
-    will start QT Application if not running yet
-
-    :returns: QApplication
-
-    """
+    """will start QT Application if not running yet and returns QApplication """
 
     # start qt event loop
     _instance = QtWidgets.QApplication.instance()
@@ -92,6 +197,7 @@ def get_qt_app():
 
 
 def get_last_path():
+    """Returns the path of the file last opened"""
     try:
         fp = open(config_path + '\\path.txt', 'r')
         path = fp.read()
@@ -105,6 +211,8 @@ def get_last_path():
 
 
 def save_path(filename):
+    """Save path of last opened file"""
+
     if len(filename) > 1:
         fp = open(config_path + '\\path.txt', 'w')
         path, fname = os.path.split(filename)
@@ -115,29 +223,9 @@ def save_path(filename):
     return path
 
 
-def set_directory():
-    path = get_last_path()
-
-    try:
-        get_qt_app()
-    except BaseException:
-        pass
-
-    options = QtWidgets.QFileDialog.Options()
-    options |= QtWidgets.QFileDialog.ShowDirsOnly
-
-    fname = str(QtWidgets.QFileDialog.getExistingDirectory(None, "Select Directory", path, options=options))
-
-    path = save_path(fname)
-
-    return path
-
-
 def savefile_dialog(initial_file='*.hf5', file_types=None):
-    """
-        Opens a save dialog in QT and returns an "*.hf5" file.
-        New now with initial file
-    """
+    """Opens a save dialog in QT and returns name of file. New now with initial file"""
+
     # Check whether QT is available
     if not QT_available:
         print('No QT dialog')
@@ -164,35 +252,33 @@ def savefile_dialog(initial_file='*.hf5', file_types=None):
 
 
 def openfile_dialog(file_types=None):  # , multiple_files=False):
-    """
-    Opens a File dialog which is used in open_file() function
-    This function uses tkinter or pyQt5.
-    The app of the Gui has to be running for QT so Tkinter is a safer bet.
-    In jupyter notebooks use %gui Qt early in the notebook.
+    """Opens a File dialog which is used in open_file() function
 
+    This function uses pyQt5.
+    The app of the Gui has to be running for QT. Tkinter does not run on Macs at this point in time.
+    In jupyter notebooks use %gui Qt early in the notebook.
 
     The file looks first for a path.txt file for the last directory you used.
 
     Parameters
     ----------
-    file_types : string of the file type filter
+    file_types : string
+        file type filter in the form of '*.hf5'
 
 
     Returns
     -------
-    filename : full filename with absolute path and extension as a string
-
-    Examples
-    --------
+    filename : string
+        full filename with absolute path and extension as a string
 
+    Example
+    -------
     >> import file_tools as ft
-    >>
     >> filename = ft.openfile_dialog()
-    >>
     >> print(filename)
 
-
     """
+
     # determine file types by extension
     if file_types is None:
         file_types = 'TEM files (*.dm3 *.qf3 *.ndata *.h5 *.hf5);;pyNSID files (*.hf5);;QF files ( *.qf3);;' \
@@ -203,77 +289,90 @@ def openfile_dialog(file_types=None):  # , multiple_files=False):
 
         # file_types = [("TEM files",["*.dm*","*.hf*","*.ndata" ]),("pyUSID files","*.hf5"),("DM files","*.dm*"),
         # ("Nion files",["*.h5","*.ndata"]),("all files","*.*")]
-    # Determine last path used
 
+    # Determine last path used
     path = get_last_path()
     _ = get_qt_app()
 
     filename = sidpy.io.interface_utils.openfile_dialog(file_types=file_types, file_path=path)
-    #
-    save_path(filename)
 
+    save_path(filename)
     return filename
 
 
-def open_file(filename=None, save_file=False, h5_group=None):
-    """
-    Opens a file if the extension is .hf5, .dm3 or .dm4
-    If no filename is provided the qt open_file windows opens
+def open_file(filename=None,  h5_group=None):  # save_file=False,
+    """Opens a file if the extension is .hf5, .ndata, .dm3 or .dm4
 
+    If no filename is provided the QT open_file windows opens (if QT_available==True)
     Everything will be stored in a NSID style hf5 file.
-
     Subroutines used:
         - NSIDReader
         - nsid.write_
             - get_main_tags
             - get_additional tags
 
+    Parameters
+    ----------
+    filename: str
+        name of file to be opened, if filename is None, a QT file dialog will try to open
+    h5_group: hd5py.Group
+        not used yet #TODO: provide hook for usage of external chosen group
+
+    Returns
+    -------
+    sidpy.Dataset
+        sidpy dataset with location of hdf5 dataset as attribute
+
     """
-    get_qt_app()
     if filename is None:
-        filename = openfile_dialog()
-        if filename == '':
-            return
+        if QT_available:
+            get_qt_app()
+            filename = openfile_dialog()
+        else:
+            raise TypeError('filename must be provided if QT is not installed')
+    else:
+        if not isinstance(filename, str):
+            raise TypeError('filename must be a non-empty string or None (to a QT open file dialog)')
+        elif filename == '':
+            raise TypeError('filename must be a non-empty string or None (to a QT open file dialog)')
+
     path, file_name = os.path.split(filename)
     basename, extension = os.path.splitext(file_name)
 
     if extension == '.hf5':
-        h5_file = h5py.File(filename, mode='a')
 
-        h5_group = get_start_channel(h5_file)
-        print()
-        if 'nDim_Data' in h5_group:
-            h5_dataset = h5_group['nDim_Data']
+        reader = pyNSID.NSIDReader(filename)
+        datasets = reader.read()
+        if len(datasets)<1:
+            print('no hdf5 dataset found in file')
+            return
+        else:
+            return datasets[0]
 
-            h5_dataset.attrs['title'] = basename
-            reader = NSIDReader(h5_dataset)
-            dataset = reader.read_h5py_dataset(h5_dataset)
-            dataset.h5_file = h5_file
-        elif 'Raw_Data' in h5_group:
+        """ should go to no dataset found
+        if 'Raw_Data' in h5_group:
             dataset = read_old_h5group(h5_group)
             dataset.h5_dataset = h5_group['Raw_Data']
-        else:
-            reader = NSIDReader(h5_file['Measurement_000/Channel_000'])
-            dataset = reader.read()[-1]
-            dataset.h5_file = h5_file
-        return dataset
+        """
+
 
     elif extension in ['.dm3', '.dm4', '.ndata', '.h5']:
 
         # tags = open_file(filename)
         if extension in ['.dm3', '.dm4']:
             reader = DM3Reader(filename)
-        elif extension in ['.ndata', '.h5']:
+        else:   # extension in ['.ndata', '.h5']:
             reader = NionReader(filename)
-        else:
-            IOError('problem')
+
         path, file_name = os.path.split(filename)
         basename, _ = os.path.splitext(file_name)
         dset = reader.read()
-        dset.title = basename.strip().replace('-', '_')
-        dset.filename = basename.strip().replace('-', '_')
-        dset.original_metadata = flatten_dict(dset.original_metadata)
 
+        if extension in ['.dm3', '.dm4']:
+            dset.title = basename.strip().replace('-', '_')
+        dset.filename = basename.strip().replace('-', '_')
+        # dset.original_metadata = flatten_dict(dset.original_metadata)
+        filename = os.path.join(path,  dset.title+extension)
         h5_filename = get_h5_filename(filename)
         h5_file = h5py.File(h5_filename, mode='a')
 
@@ -282,10 +381,10 @@ def open_file(filename=None, save_file=False, h5_group=None):
         else:
             if not isinstance(h5_group, h5py.Group):
                 h5_group = h5_file.create_group('Measurement_000/Channel_000')
-            dset.axes = dset._axes
-            dset.attrs = {}
+            # dset.axes = dset._axes
+            # dset.attrs = {}
             h5_dataset = pyNSID.hdf_io.write_nsid_dataset(dset, h5_group)
-            dset.original_metadata = nest_dict(dset.original_metadata)
+            # dset.original_metadata = nest_dict(dset.original_metadata)
 
             dset.h5_dataset = h5_dataset
         return dset
@@ -295,6 +394,8 @@ def open_file(filename=None, save_file=False, h5_group=None):
 
 
 def get_h5_filename(fname):
+    """Determines file name of hdf5 file for newly converted data file"""
+
     path, filename = os.path.split(fname)
     basename, extension = os.path.splitext(filename)
     h5_file_name_original = os.path.join(path, basename + '.hf5')
@@ -314,10 +415,15 @@ def get_h5_filename(fname):
 
 
 def get_start_channel(h5_file):
+    """ Legacy for get start channel"""
+
+    DeprecationWarning('Depreciated: use function get_main_channel instead')
     return get_main_channel(h5_file)
 
 
 def get_main_channel(h5_file):
+    """Returns name of first channel group in hdf5-file"""
+
     current_channel = None
     if 'Measurement_000' in h5_file:
         if 'Measurement_000/Channel_000' in h5_file:
@@ -325,25 +431,57 @@ def get_main_channel(h5_file):
     return current_channel
 
 
-def h5_tree(input):
-    """
-    Just a wrapper for the sidpy function print_tree,
+def h5_tree(input_object):
+    """Just a wrapper for the sidpy function print_tree,
+
     so that sidpy does not have to be loaded in notebook
+
     """
-    if isinstance(input, sidpy.Dataset):
-        if not isinstance(input.h5_dataset, h5py.Dataset):
+
+    if isinstance(input_object, sidpy.Dataset):
+        if not isinstance(input_object.h5_dataset, h5py.Dataset):
             raise ValueError('sidpy dataset does not have an associated h5py dataset')
-        h5_file = input.h5_dataset.file
-    elif isinstance(input, h5py.Dataset):
-        h5_file = input.file
-    elif isinstance(input, (h5py.Group, h5py.File)):
-        h5_file = input
+        h5_file = input_object.h5_dataset.file
+    elif isinstance(input_object, h5py.Dataset):
+        h5_file = input_object.file
+    elif isinstance(input_object, (h5py.Group, h5py.File)):
+        h5_file = input_object
     else:
         raise TypeError('should be a h5py.object or sidpy Dataset')
     sidpy.hdf_utils.print_tree(h5_file)
 
 
 def log_results(h5_group, dataset=None, attributes=None):
+    """Log Results in hdf5-file
+
+    Saves either a sidpy.Dataset or dictionary in a hdf5-file.
+    The group for the result will consist of 'Log_ and a running index.
+    That group will be placed in h5_group.
+
+    Parameters
+    ----------
+    h5_group: hd5py.Group
+        groups where result group are to be stored
+    dataset: sidpy.Dataset or None
+        sidpy dataset to be stored
+    attributes: dict
+        dictionary containing results that are not based on a sidpy.Dataset
+
+    Returns
+    -------
+    log_group: hd5py.Group
+        group in hdf5 file with results.
+
+    """
+    if isinstance(h5_group, sidpy.Dataset):
+        h5_group = h5_group.h5_dataset
+        if not isinstance(h5_group, h5py.Dataset):
+            raise TypeError('Use h5_dataset of sidpy.Dataset is not a valid h5py.Dataset')
+        h5_group = h5_group.parent.parent
+
+    if not isinstance(h5_group, h5py.Group):
+        raise TypeError('Need a valid h5py.Group for logging results')
+
     if dataset is None:
         log_group = sidpy.hdf.prov_utils.create_indexed_group(h5_group, 'Log_')
     else:
@@ -356,31 +494,70 @@ def log_results(h5_group, dataset=None, attributes=None):
             if 'analysis' in dataset.meta_data:
                 log_group['analysis'] = dataset.meta_data['analysis']
 
-        dataset.h5_dataset = log_group[dataset.title.replace('-', '_')]
+        dataset.h5_dataset = log_group[dataset.title.replace('-', '_')][dataset.title.replace('-', '_')]
     if attributes is not None:
         for key, item in attributes.items():
             if key not in log_group:
                 log_group[key] = item
-
     return log_group
 
 
+###
+# Crystal Structure Read and Write
+###
+def h5_add_crystal_structure(h5_file, crystal_tags):
+    structure_group = sidpy.hdf.prov_utils.create_indexed_group(h5_file, 'Structure_')
+
+    structure_group['unit_cell'] = crystal_tags['unit_cell']
+    structure_group['relative_positions'] = crystal_tags['base']
+    structure_group['title'] = str(crystal_tags['crystal_name'])
+    structure_group['_' + crystal_tags['crystal_name']] = str(crystal_tags['crystal_name'])
+    structure_group['elements'] = np.array(crystal_tags['elements'], dtype='S')
+    if 'zone_axis' in structure_group:
+        structure_group['zone_axis'] = np.array(crystal_tags['zone_axis'], dtype=float)
+    else:
+        structure_group['zone_axis'] = np.array([1., 0., 0.], dtype=float)
+    h5_file.flush()
+    return structure_group
+
+
+def h5_get_crystal_structure(structure_group):
+    crystal_tags = {}
+    crystal_tags['unit_cell'] = structure_group['unit_cell'][()]
+    crystal_tags['base'] = structure_group['relative_positions'][()]
+    crystal_tags['crystal_name'] = structure_group['title'][()]
+    if '2D' in structure_group:
+        crystal_tags['2D'] = structure_group['2D'][()]
+    elements = structure_group['elements'][()]
+    crystal_tags['elements'] = []
+    for e in elements:
+        crystal_tags['elements'].append(e.astype(str, copy=False))
+
+    if 'zone_axis' in structure_group:
+        crystal_tags['zone_axis'] = structure_group['zone_axis'][()]
+    return crystal_tags
+
+
 ###############################################
 # Support old pyTEM file format
 ###############################################
 
 def read_old_h5group(current_channel):
-    """
-    make a  sidpy dataset from pyUSID style hdf5 group
-    input
+    """Make a  sidpy.Dataset from pyUSID style hdf5 group
+
+    Parameters
+    ----------
         current_channel: h5_group
-    return
-        sidpy Dataset
+
+    Returns
+    -------
+        sidpy.Dataset
     """
+
     dim_dir = []
     if 'nDim_Data' in current_channel:
         h5_dataset = current_channel['nDim_Data']
-        reader = NSIDReader(h5_dataset)
+        reader = pyNSID.NSIDReader(h5_dataset)
         dataset = reader.read_h5py_dataset(h5_dataset)
         dataset.h5_file = current_channel.file
         return dataset
@@ -417,13 +594,13 @@ def read_old_h5group(current_channel):
 
     if 'SPATIAL' in dim_dir:
         if 'SPECTRAL' in dim_dir:
-            sid_dataset.data_type = sidpy.DataTypes.SPECTRAL_IMAGE
+            sid_dataset.data_type = sidpy.DataType.SPECTRAL_IMAGE
         elif 'TEMPORAL' in dim_dir:
-            sid_dataset.data_type = sidpy.DataTypes.IMAGE_STACK
+            sid_dataset.data_type = sidpy.DataType.IMAGE_STACK
         else:
-            sid_dataset.data_type = sidpy.DataTypes.IMAGE
+            sid_dataset.data_type = sidpy.DataType.IMAGE
     else:
-        sid_dataset.data_type = sidpy.DataTypes.SPECTRUM
+        sid_dataset.data_type = sidpy.DataType.SPECTRUM
 
     sid_dataset.quantity = 'intensity'
     sid_dataset.units = 'counts'
@@ -436,14 +613,15 @@ def read_old_h5group(current_channel):
 
 
 def set_dimensions(dset, current_channel):
-    """
-    Attaches correct dimension from old pyTEMlib style.
-    Input:
-    dset: sidpy Dataset
-    current_channel: hdf5 group
+    """Attaches correct dimension from old pyTEMlib style.
+
+    Parameters
+    ----------
+    dset: sidpy.Dataset
+    current_channel: hdf5.Group
     """
     dim = 0
-    if dset.data_type == sidpy.DataTypes.IMAGE_STACK:
+    if dset.data_type == sidpy.DataType.IMAGE_STACK:
         dset.set_dimension(dim, sidpy.Dimension(np.arange(dset.shape[dim]), name='frame',
                                                 units='frame', quantity='stack',
                                                 dimension_type='TEMPORAL'))
@@ -472,7 +650,7 @@ def set_dimensions(dset, current_channel):
                                                 units=units_x, quantity='Length',
                                                 dimension_type='SPATIAL'))
         dim += 1
-    if dset.data_type in [sidpy.DataTypes.SPECTRUM, sidpy.DataTypes.SPECTRAL_IMAGE]:
+    if dset.data_type in [sidpy.DataType.SPECTRUM, sidpy.DataType.SPECTRAL_IMAGE]:
         if 'spectral_scale_x' in current_channel:
             scale_s = current_channel['spectral_scale_x'][()]
         else:
