@@ -51,7 +51,7 @@ from sklearn.utils.extmath import randomized_svd
 _SimpleITK_present = True
 try:
     import SimpleITK as sITK
-except ModuleNotFoundError:
+except ImportError:
     _SimpleITK_present = False
 
 if not _SimpleITK_present:
@@ -82,7 +82,7 @@ def fourier_transform(dset):
 
         Input
         -----
-        dset: sidp Dataset
+        dset: sidpy.Dataset
 
         Usage
         -----
@@ -96,12 +96,12 @@ def fourier_transform(dset):
     selection = []
     image_dim = []
     # image_dim = get_image_dims(sidpy.DimensionTypes.SPATIAL)
-    if dset.data_type == sidpy.DataTypes.IMAGE_STACK:
+    if dset.data_type == sidpy.DataType.IMAGE_STACK:
         for dim, axis in dset._axes.items():
-            if axis.dimension_type == sidpy.DimensionTypes.SPATIAL:
+            if axis.dimension_type == sidpy.DimensionType.SPATIAL:
                 selection.append(slice(None))
                 image_dim.append(dim)
-            elif axis.dimension_type == sidpy.DimensionTypes.TEMPORAL or len(dset) == 3:
+            elif axis.dimension_type == sidpy.DimensionType.TEMPORAL or len(dset) == 3:
                 selection.append(slice(None))
                 stack_dim = dim
             else:
@@ -110,7 +110,7 @@ def fourier_transform(dset):
             raise ValueError('need at least two SPATIAL dimension for an image stack')
         image_stack = np.squeeze(np.array(dset)[selection])
         image = np.sum(np.array(image_stack), axis=stack_dim)
-    elif dset.data_type == sidpy.DataTypes.IMAGE:
+    elif dset.data_type == sidpy.DataType.IMAGE:
         image = np.array(dset)
     else:
         return
@@ -209,7 +209,7 @@ def diffractogram_spots(dset, spot_threshold):
     rec_scale = np.array([get_slope(dset.u.values), get_slope(dset.v.values), 1])
 
     # spot detection ( for future referece there is no symmetry assumed here)
-    data = np.array(dset).T
+    data = np.array(np.log(1+np.abs(dset))).T
     data = (data - data.min())
     data = data/data.max()
     # some images are strange and blob_log does not work on the power spectrum
@@ -218,7 +218,7 @@ def diffractogram_spots(dset, spot_threshold):
     except ValueError:
         spots_random = (peak_local_max(np.array(data.T), min_distance=3, threshold_rel=spot_threshold) - center[:2]) \
                        * rec_scale
-        spots_random = np.hstack(spots_random,np.zeros((spots_random.shape[0],1)))
+        spots_random = np.hstack(spots_random, np.zeros((spots_random.shape[0], 1)))
 
     print(f'Found {spots_random.shape[0]} reflections')
 
@@ -296,16 +296,18 @@ def rotational_symmetry_diffractogram(spots):
 #####################################################
 
 
-def complete_registration(main_dataset):
+def complete_registration(main_dataset, storage_channel=None):
+    """Rigid and then non-rigid (demon) registration"""
     rigid_registered_dataset = rigid_registration(main_dataset)
-    current_channel = main_dataset.h5_dataset.parent
-    registration_channel = log_results(current_channel, rigid_registered_dataset)
+    if storage_channel is None:
+        storage_channel = main_dataset.h5_dataset.parent.parent
+
+    registration_channel = log_results(storage_channel, rigid_registered_dataset)
 
     print('Non-Rigid_Registration')
 
     non_rigid_registered = demon_registration(rigid_registered_dataset)
-
-    registration_channel = log_results(current_channel, non_rigid_registered)
+    registration_channel = log_results(storage_channel, non_rigid_registered)
 
     return non_rigid_registered, rigid_registered_dataset
 
@@ -313,13 +315,19 @@ def complete_registration(main_dataset):
 def demon_registration(dataset, verbose=False):
     """
     Diffeomorphic Demon Non-Rigid Registration
+
     Usage:
     ------
     dem_reg = demon_reg(cube, verbose = False)
 
-    Input:
-        cube: stack of image after rigid registration and cropping
-    Output:
+    Parameters
+    ----------
+    dataset: sidpy.Dataset
+        stack of image after rigid registration and cropping
+    verbose: boolean
+        optional for increased output
+    Returns
+    -------
         dem_reg: stack of images with non-rigid registration
 
     Depends on:
@@ -388,9 +396,10 @@ def demon_registration(dataset, verbose=False):
     demon_registered.source = dataset.title
 
     demon_registered.metadata = {'analysis': 'non-rigid demon registration'}
-    if 'boundaries' in dataset.metadata:
-        demon_registered.metadata['boundaries'] = dataset.metadata['boundaries']
-
+    if 'input_crop' in dataset.metadata:
+        demon_registered.metadata['input_crop'] = dataset.metadata['input_crop']
+    if 'input_shape' in dataset.metadata:
+        demon_registered.metadata['input_shape'] = dataset.metadata['input_shape']
     return demon_registered
 
 
@@ -403,9 +412,14 @@ def rigid_registration(dataset):
     used phase_cross_correlation from skimage.registration
     (we determine drift from one image to next)
 
-    Input hdf5 group with image_stack dataset
+    Parameters
+    ----------
+    dataset: sidpy.Datset
+        sidpy dataset with image_stack dataset
 
-    Output Registered Stack and drift (with respect to center image)
+    Returns
+    -------
+    Registered Stack and drift (with respect to center image)
 
     """
 
@@ -449,12 +463,13 @@ def rigid_registration(dataset):
         progress.close()
     rig_reg, drift = rig_reg_drift(dataset, relative_drift)
 
-    crop_reg, boundaries = crop_image_stack(rig_reg, drift)
+    crop_reg, input_crop = crop_image_stack(rig_reg, drift)
 
     rigid_registered = dataset.like_data(crop_reg)
     rigid_registered.title = 'Rigid Registration'
     rigid_registered.source = dataset.title
-    rigid_registered.metadata = {'analysis': 'rigid sub-pixel registration', 'drift': drift, 'boundaries': boundaries}
+    rigid_registered.metadata = {'analysis': 'rigid sub-pixel registration', 'drift': drift,
+                                 'input_crop': input_crop, 'input_shape': dataset.shape[1:]}
 
     return rigid_registered
 
@@ -570,8 +585,6 @@ class ImageWithLineProfile:
 
 def histogram_plot(image_tags):
     nbins = 75
-    minbin = 0.
-    maxbin = 1.
     color_map_list = ['gray', 'viridis', 'jet', 'hot']
     
     if 'minimum_intensity' not in image_tags: 
@@ -826,8 +839,9 @@ def align_crystal_reflections(spots, crystals):
         
     return crystal_reflections_polar, angles
 
+
 # Deconvolution
-def decon_lr(o_image, probe, tags, verbose=False):
+def decon_lr(o_image, probe,  verbose=False):
     """
     # This task generates a restored image from an input image and point spread function (PSF) using
     # the algorithm developed independently by Lucy (1974, Astron. J. 79, 745) and Richardson
@@ -871,23 +885,17 @@ def decon_lr(o_image, probe, tags, verbose=False):
 
     response_ft = fftpack.fft2(probe_c)
 
-    if 'ImageScanned' in tags:
-        ab = tags['ImageScanned']
-    elif 'aberrations' in tags:
-        ab = tags['aberrations']
-    if 'convAngle' not in ab:
-        ab['convAngle'] = 30
-    ap_angle = ab['convAngle'] / 1000.0
+    ap_angle = o_image.metadata['experiment']['convergence_angle'] / 1000.0  # now in rad
 
-    e0 = float(ab['EHT'])
+    e0 = float(o_image.metadata['experiment']['acceleration_voltage'])
 
     wl = get_wavelength(e0)
-    ab['wavelength'] = wl
+    o_image.metadata['experiment']['wavelength'] = wl
 
     over_d = 2 * ap_angle / wl
 
-    dx = tags['pixel_size']
-    dk = 1.0 / float(tags['fov'])
+    dx = o_image.x[1]-o_image.x[0]
+    dk = 1.0 / float( o_image.x[-1])  # last value of x axis is field of view
     screen_width = 1 / dx
 
     aperture = np.ones(o_image.shape, dtype=np.complex64)
@@ -903,7 +911,7 @@ def decon_lr(o_image, probe, tags, verbose=False):
 
     tp1 = t_xv ** 2 + t_yv ** 2 >= app_ratio ** 2
     aperture[tp1.T] = 0.
-    print(app_ratio, screen_width, dk)
+    # print(app_ratio, screen_width, dk)
 
     if QT_available:
         progress = ProgressDialog("Lucy-Richardson", 100)
@@ -941,16 +949,15 @@ def decon_lr(o_image, probe, tags, verbose=False):
                 count = 0
             progress.set_value(count)
 
-
-        if i > 1000:
+        if i > 500:
             dest = 0.0
             print('terminate')
     if QT_available:
         progress.close()
     print('\n Lucy-Richardson deconvolution converged in ' + str(i) + '  Iterations')
     est2 = np.real(fftpack.ifft2(fftpack.fft2(est) * fftpack.fftshift(aperture)))
-    # plt.imshow(np.real(np.log10(np.abs(fftpack.fftshift(fftpack.fft2(est)))+1)+aperture), origin='lower',)
-    # plt.show()
     print(est2.shape)
-    return est2
-
+    out_dataset =  o_image.like_data(est2)
+    out_dataset.title = 'Lucy Richardson deconvolution'
+    out_dataset.data_type = 'image'
+    return out_dataset
