@@ -1,10 +1,6 @@
 #!/usr/bin/env python
 # -*- coding: iso-8859-1 -*-
-"""
-Python class for reading Nion Swift files into sidpy Dataset
 
-will move to Scope Reader in pycroscopy
-"""
 ################################################################################
 # Python class for reading Nion Swift files into sidpy Dataset
 # and extracting all metadata
@@ -167,14 +163,14 @@ class NionReader(sidpy.Reader):
                 self.original_metadata = json.loads(json_properties)
 
         self.get_dimensions()
-        ## Need to switch image dimensions in Nion format
+        # Need to switch image dimensions in Nion format
         image_dims = []
         for dim, axis in enumerate(self.dimensions):
-            print(dim, axis)
-            if axis.dimension_type == sidpy.DimensionTypes.SPATIAL:
+            # print(dim, axis)
+            if axis.dimension_type == sidpy.DimensionType.SPATIAL:
                 image_dims.append(dim)
-        print('image_dims', image_dims)
-        print(self.data_cube.shape)
+        # print('image_dims', image_dims)
+        # print(self.data_cube.shape)
         if len(image_dims) == 2:
             self.data_cube = np.swapaxes(self.data_cube, image_dims[0], image_dims[1])
             temp = self.dimensions[image_dims[0]].copy()
@@ -183,6 +179,7 @@ class NionReader(sidpy.Reader):
 
         dataset = sidpy.Dataset.from_array(self.data_cube)
 
+        dim_names = []
         for dim, axis in enumerate(self.dimensions):
             dataset.set_dimension(dim, axis)
 
@@ -219,7 +216,7 @@ class NionReader(sidpy.Reader):
 
         spectral_dim = False
         for axis in dataset._axes.values():
-            if axis.dimension_type == sidpy.DimensionTypes.SPECTRAL:
+            if axis.dimension_type == sidpy.DimensionType.SPECTRAL:
                 spectral_dim = True
 
         if len(dataset.shape) > 3:
@@ -230,24 +227,24 @@ class NionReader(sidpy.Reader):
             else:
                 dataset.data_type = 'IMAGE_STACK'
                 for dim, axis in dataset._axes.items():
-                    if axis.dimension_type != sidpy.DimensionTypes.SPATIAL:
+                    if axis.dimension_type != sidpy.DimensionType.SPATIAL:
                         dataset.set_dimension(dim, sidpy.Dimension(axis.values,
                                                                    name='frame',
                                                                    units='frame',
                                                                    quantity='stack',
-                                                                   dimension_type=sidpy.DimensionTypes.TEMPORAL))
+                                                                   dimension_type=sidpy.DimensionType.TEMPORAL))
                         break
 
         elif len(dataset.shape) == 2:
             if spectral_dim:
-                dataset.data_type = sidpy.DataTypes.SPECTRAL_IMAGE
+                dataset.data_type = sidpy.DataType.SPECTRAL_IMAGE
             else:
-                dataset.data_type = sidpy.DataTypes.IMAGE
+                dataset.data_type = sidpy.DataType.IMAGE
         elif len(dataset.shape) == 1:
             if spectral_dim:
-                dataset.data_type = sidpy.DataTypes.SPECTRUM
+                dataset.data_type = sidpy.DataType.SPECTRUM
             else:
-                dataset.data_type = sidpy.DataTypes.LINE_PLOT
+                dataset.data_type = sidpy.DataType.LINE_PLOT
 
     def get_dimensions(self):
         dic = self.original_metadata
@@ -260,7 +257,7 @@ class NionReader(sidpy.Reader):
             for dim in range(len(dic['dimensional_calibrations'])):
                 dimension_tags = dic['dimensional_calibrations'][dim]
                 units = dimension_tags['units']
-                values = (np.arange(self.data_cube.shape[int(dim)]) - dimension_tags['offset']) * dimension_tags['scale']
+                values = (np.arange(self.data_cube.shape[int(dim)])-dimension_tags['offset']) * dimension_tags['scale']
 
                 if 'eV' == units:
                     self.dimensions.append(sidpy.Dimension(values, name='energy_loss', units=units,
@@ -270,17 +267,17 @@ class NionReader(sidpy.Reader):
                                                            quantity='energy', dimension_type='spectral'))
                 elif '1/' in units or units in ['mrad', 'rad']:
                     self.dimensions.append(sidpy.Dimension(values, name=reciprocal_name, units=units,
-                                                           quantity='reciprocal distance',dimension_type='reciprocal'))
+                                                           quantity='reciprocal distance',
+                                                           dimension_type='reciprocal'))
                     reciprocal_name = chr(ord(reciprocal_name) + 1)
                 elif 'nm' in units:
                     self.dimensions.append(sidpy.Dimension(values, name=spatial_name, units=units,
                                                            quantity='distance', dimension_type='spatial'))
                     spatial_name = chr(ord(spatial_name) + 1)
                 else:
-                    self.dimensions.append(sidpy.Dimension(values, name='generic', units='generic',
+                    self.dimensions.append(sidpy.Dimension(values, name=f'generic_{dim}', units='generic',
                                                            quantity='generic', dimension_type='UNKNOWN'))
 
-
     def get_filename(self):
         return self.__filename
 
